/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_GUI_MODULE_HPP
#define FGGL_GUI_MODULE_HPP

#include "fggl/gui/fonts.hpp"
#include "fggl/gui/model/parser.hpp"

#include "fggl/data/module.hpp"
#include "fggl/assets/packed/module.hpp"

namespace fggl::gui {

	constexpr auto MIME_TTF = assets::from_mime("font/ttf");
	constexpr auto ASSET_FONT_TTF = assets::make_asset_type("font/ttf");

	static assets::AssetTypeID is_font(std::filesystem::path path) {
		if ( path.extension() == ".ttf" ){
			return ASSET_FONT_TTF;
		}
		return assets::INVALID_ASSET_TYPE;
	}

	struct FreeType {
		constexpr static const char *name = "fggl::gui::FreeType";
		constexpr static const std::array<modules::ModuleService, 2> provides = {
			FontLibrary::service,
			model::WidgetFactory::service
		};
		constexpr static const std::array<modules::ModuleService, 2> depends = {
			data::Storage::service,
			assets::CheckinAdapted::service
		};
		static bool factory(modules::ModuleService name, modules::Services &serviceManager);
	};

	bool FreeType::factory(modules::ModuleService service, modules::Services &services) {
		if (service == FontLibrary::service) {
			auto storage = services.get<data::Storage>();
			services.create<FontLibrary>(storage);

			auto* checkin = services.get<assets::CheckinAdapted>();
			checkin->setLoader( MIME_TTF, assets::NEEDS_CHECKIN, is_font );

			return true;
		}

		if ( service == model::WidgetFactory::service ) {
			auto fonts = services.get<FontLibrary>();
			services.create<model::WidgetFactory>(fonts);
			return true;
		}

		return false;
	}
}

#endif //FGGL_GUI_MODULE_HPP
